#include "cthugha.h"
#include "display.h"
#include "options.h"
#include "keys.h"
#include "action.h"
#include "information.h"
#include "sound.h"
#include "translate.h"
#include "interface.h"
#include "disp-sys.h"

#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* possible display-function */
int screen_up();	int screen_down();	int screen_2hor();
int screen_r2hor();	int screen_4hor();	int screen_2verd();
int screen_r2verd();	int screen_4kal();	int screen_mirrorH();
int screen_mirrorHV();	int screen_mirrorV();	int screen_sheet();

feature_entry _screens[] = {
    { screen_up,	1,	"Up",		"Up Display" },
    { screen_down,	1,	"Down",		"Upside Down"},
    { screen_2hor,	1,	"2hor",		"Hor. Split out"},
    { screen_r2hor,	1,	"r2hor",	"Hor. Split in"},
    { screen_4hor,	1,	"4hor",		"Kaleidoscope"},
    { screen_2verd,	1,	"2verd",	"90deg rot. mirror"},
    { screen_r2verd,	1,	"r2verd",	"90deg rot. mirror II"},
    { screen_4kal,	1,	"4kal",		"90deg Kaleidoscope"},
    { screen_mirrorH,	1,	"mirrorH",	"Mirror horizontal"},
    { screen_mirrorHV,	1,	"mirrorHV",	"Mirror horiz&vert"},
    { screen_mirrorV,	1,	"mirrorV",	"Mirror vertical"},
    { screen_sheet,	1,	"sheet",	"A Sheet of Paper"}
};
feature screens = {
    "display",
    _screens,
    sizeof(_screens) / sizeof(feature_entry),
    CHANGE_RANDOM,
    change_update_screen
};
xy screen_sizes[] = {
    {1,1},	/* up */	{1,1},	/* down */
    {1,1},	/* 2hor */	{1,1},	/* r2hor */
    {1,1},	/* 4hor */	{1,1},	/* 2verd */
    {1,1},	/* r2verd */	{1,1},	/* 4 kal */
    {2,1},	/* mirrorH */	{2,2},	/* mirrorHV */
    {1,2},	/* mirrorV */   {2,2}   /* sheet */
};

int (*update_screen)(void);			/* pointer to display-funct. */

xy disp_do_tiles = {1,1};			/* do tiles or not */
xy disp_nr_tiles = {0,0};			/* nr of tiles to do */
xy disp_size = {0,0};				/* size of drawing area */
xy disp_one_size = {0,0};
xy draw_size = {0,0};				/* size of the drawn image */

int bytes_per_line = 0;

char * buff1;					/* Screen-Memory 1 */
char * buff2;					/* Screen-Memory 2 */

unsigned char * active_buffer;			/* Next on screen */
unsigned char * passive_buffer;			/* right now on screen */

char screen_first[256] = "";			/* Start with this scrn-fkt */

int display_frames = 0;				/* counter for speed-tests */
time_t display_start;

int display_clear = 1;				/* clear screen */

int tiles_x = 1, tiles_y = 1;			/* how many tiles */
int display_mode = 0;				/* graphic mode */

int bpp = 8;					/* bits per pixel */
int bypp = 1;					/* bytes per pixel */

unsigned char * display_mem = NULL;		/* memory for screen-fkt
						   to draw to */

enum draw_mode_t draw_mode = DM_direct;		/* how drawing is done */



/*
 * Initialization of display 
 *  - load palettes and PCX 
 *  - allocate buffer
 *  - does not switch to graph mode
 */
int init_display() {
  
    if( init_display_sys() )
	return 1;

    /* allocate memory for buffers */
    buff1 = cth_memory(NULL, BUFF_SIZE+6*BUFF_WIDTH,
		       "Can not allocate memory for buffer 1.\n");
    buff2 = cth_memory(NULL, BUFF_SIZE+6*BUFF_WIDTH,
		       "Can not allocate memory for buffer 2.\n");

    /* The buffer has a border of 3 lines on top on bottom. These
       lines are set black before the 'flame' function is called.
       The border is not displayed to the screen
       */
    active_buffer  = buff1 + 3*BUFF_WIDTH;
    passive_buffer = buff2 + 3*BUFF_WIDTH;
	
    /* clear active buffer */
    bzero(active_buffer, BUFF_SIZE);

    /* initiaize (load) the palettes (internal and external) */
    if( load_palettes() )
	return 1;
		
    /* load the pcx-files */
    if ( init_pcx() )
	return 1;

    /* check if any palettes in use */
    if ( palettes.nr_entries == 0) {
	printfe("No palettes specified.\n");
	return 1;
    }

    return 0;
}

/*
 * Clean upt display
 */
int exit_display() {
    exit_graph_mode();
    return 0;
}


/*
 * Turn on graph-mode
 *   - turnon graph mode
 *   - set X11 palette
 *   - allocate XImage
 */
int init_graph_mode() {

    if( init_graph_mode_sys() )
	return 1;

    init_palettes();				/* really set first palette */

    printfv(3, "display size: %dx%d\n", disp_size.x, disp_size.y);
    printfv(3, "bytes/line  : %d\n", bytes_per_line);
    printfv(3, "bytes/pixel : %d\n", bypp);

    display_start = time(0);
	
    return 0;
}

/*
 * Switch back to text mode
 */
int exit_graph_mode() {
    exit_graph_mode_sys();
    exit_palettes();
    return 0;
}


/*
 * Change/Select how buffer is displayed on screen
 * passes only the parameters to 'opt_change' to display the menu
 */
int change_update_screen(int to) {
    update_screen = feature_change(to, &screens)->data;
    update_status();
    display_clear = 1;
    return 0;
}
int select_screen() {
    selection(&screens, "Select Screens\n",  change_update_screen);
    return 0;
}


/*****************************************************************************/

/*
 * exchange active and passive buffer
 */
void swap_buffers() {
    char * tmp;
    tmp = active_buffer; active_buffer = passive_buffer; passive_buffer = tmp;
}



/*
 * calculate information for tile and offset 
 */
int update_draw_size() {
    xy s;

    s = screen_sizes[screens.current];

    disp_nr_tiles.x = disp_do_tiles.x ? (disp_size.x/BUFF_WIDTH/s.x) : 1;
    disp_nr_tiles.y = disp_do_tiles.y ? (disp_size.y/BUFF_HEIGHT/s.y) : 1;

    disp_one_size.x = s.x * BUFF_WIDTH;
    disp_one_size.y = s.y * BUFF_HEIGHT;

    draw_size.x = disp_nr_tiles.x * disp_one_size.x;
    draw_size.y = disp_nr_tiles.y * disp_one_size.y;

    return 0;
}

/*
 * clear border around display if necessary 
 */
static int clear_border() {
    if ( display_text_on_screen || display_clear ) {
	disp_clear_box_sys(0,0, bytes_per_line, SCREEN_OFFSET_Y);
	disp_clear_box_sys(0, SCREEN_OFFSET_Y, SCREEN_OFFSET_X, BUFF_HEIGHT);
	disp_clear_box_sys(disp_size.x-SCREEN_OFFSET_X, SCREEN_OFFSET_Y,
			   SCREEN_OFFSET_X, BUFF_HEIGHT);
	disp_clear_box_sys(0,disp_size.y-SCREEN_OFFSET_Y,
			   bytes_per_line, SCREEN_OFFSET_Y);

	display_clear = 0;
	return 1;
    }
    return 0;
}

/*
 * tile buffer to screen
 */
int tile_buffer() {
    int i;

    for(i=1; i < disp_nr_tiles.x; i++)
	disp_copy_box_sys(SCREEN_OFFSET_X, SCREEN_OFFSET_Y,
			  disp_one_size.x, disp_one_size.y,
			  SCREEN_OFFSET_X + i*disp_one_size.x,
			  SCREEN_OFFSET_Y);

    for(i=1; i < disp_nr_tiles.y; i++)
    	disp_copy_box_sys(SCREEN_OFFSET_X, SCREEN_OFFSET_Y,
			  draw_size.x, disp_one_size.y,
			  SCREEN_OFFSET_X, 
			  SCREEN_OFFSET_Y + i*disp_one_size.y);
    return 0;
}

/*
 * Bring the active Buffer to screen.
 */
int display() {
    int clear;

    char * display_base = pre_draw_sys();

    do {
	update_draw_size();
	display_mem = display_base + SCREEN_OFFSET;
    } while( update_screen() );			/* draw the buffer */

    clear = clear_border();
    tile_buffer();
    show_text();

    disp_copy_sys(clear);

    swap_buffers();
    display_frames ++;
    
    return 0;
}

/*****************************************************************************
 * Helping macros and fuctions for the Screen-functions
 ****************************************************************************/

#define memeq(dest,src)	    *(dest)=(src)
#define tbleq1(dest,src)    *((unsigned char*)(dest))  = bitmap_colors[(src)] 
#define tbleq2(dest,src)    *((unsigned short*)(dest)) = bitmap_colors[(src)] 
#define tbleq4(dest,src)    *((unsigned int*)(dest))   = bitmap_colors[(src)] 
								
inline void tblcpy1(unsigned char * dst, unsigned char * src, int n) {
    for(;n>0;n--) {
	tbleq1(dst,*src);
	src ++; dst ++;
    }
}
inline void tblcpy2(unsigned char * dst, unsigned char * src, int n) {
    for(;n>0;n--) {
	tbleq2(dst,*src);
	src ++; dst +=2;
    }
}
inline void tblcpy4(unsigned char * dst, unsigned char * src, int n) {
    for(;n>0;n--) {
	tbleq4(dst,*src);
	src ++; dst +=4;
    }
}			    

#define SCRN(scrn)							\
    switch(draw_mode) {							\
    case DM_direct: scrn_##scrn(memcpy,  memeq,  1 ) break;		\
    case DM_mapped1: case DM_tmp_mapped:				\
	scrn_##scrn(tblcpy1, tbleq1, 1 ) break;				\
    case DM_mapped2:	scrn_##scrn(tblcpy2, tbleq2, 2 ) break;		\
    case DM_mapped4:	scrn_##scrn(tblcpy4, tbleq4, 4 ) break;		\
}

/*****************************************************************************
 * Screen-functions
 ****************************************************************************/

#define scrn_up(cpy,eq,bytes_per_pixel)			\
    for(i=BUFF_HEIGHT; i != 0; i--) {			\
	##cpy( scrn, tmp, BUFF_WIDTH);			\
	scrn += bytes_per_line;				\
	tmp += BUFF_WIDTH;				\
    }
int screen_up() {
    int i;
    unsigned char * scrn = display_mem;
    unsigned char * tmp = active_buffer;
		
    SCRN(up)
    return 0;
}


#define scrn_down(cpy,eq,bytes_per_pixel)		\
    for(i=BUFF_HEIGHT; i != 0; i--) {			\
	##cpy( scrn, tmp, BUFF_WIDTH);			\
	tmp += BUFF_WIDTH;				\
	scrn -= bytes_per_line;				\
    }
int screen_down() {
    int i;
    unsigned char * scrn = display_mem + bytes_per_line * (BUFF_HEIGHT-1);
    unsigned char * tmp = active_buffer;
    SCRN(down)
    return 0;
}


#define scrn_2hor_a(cpy,eq,bytes_per_pixel)		\
    for(i=BUFF_HEIGHT/2; i != 0; i--) {			\
	##cpy( scrn, tmp, BUFF_WIDTH);			\
	scrn += bytes_per_line;				\
	tmp += BUFF_WIDTH;				\
    }
#define scrn_2hor_b(cpy,eq,bytes_per_pixel)		\
    for(i=BUFF_HEIGHT/2; i != 0; i--) {			\
	##cpy( scrn, tmp, BUFF_WIDTH);			\
	scrn -= bytes_per_line;				\
	tmp += BUFF_WIDTH;				\
    }
int screen_2hor() {
    unsigned char * tmp, * scrn;
    int i;
	
    /* lower half of buffer maps to upper half of screen */
    tmp = active_buffer + BUFF_SIZE/2;
    scrn = display_mem;
    SCRN(2hor_a)
				
    /* lower half of buffer get turned around to lower half of screen*/
    tmp = active_buffer + BUFF_SIZE/2;
    scrn = display_mem + bytes_per_line*(BUFF_HEIGHT-1);
    SCRN(2hor_b)

    return 0;
}

		
int screen_r2hor() {
    unsigned char * tmp, * scrn;
    int i;
	
    /* lower half of buffer maps to lower half of screen */
    tmp = active_buffer + BUFF_SIZE/2;
    scrn = display_mem + BUFF_HEIGHT/2*bytes_per_line;
    SCRN(2hor_a)
		
    /* lower half of buffer get turned around to upper half of screen*/
    tmp = active_buffer + BUFF_SIZE/2;
    scrn = display_mem + bytes_per_line*(BUFF_HEIGHT/2-1);
    SCRN(2hor_b)
    
    return 0;
}


#define scrn_4hor_a(cpy,eq,bytes_per_pixel)			\
    for(y=BUFF_HEIGHT/2; y != 0; y--) {				\
								\
	/* left half */						\
	##cpy( scrn, tmp, BUFF_WIDTH/2);				\
	scrn += BUFF_WIDTH/2*bytes_per_pixel;			\
	tmp += BUFF_WIDTH/2;					\
								\
	/* right half */					\
	for(x=BUFF_WIDTH/2; x != 0; x--)	{			\
	    ##eq(scrn, *tmp);					\
	    scrn += bytes_per_pixel;				\
	    tmp --;						\
	}							\
	tmp += BUFF_WIDTH;					\
	scrn += bytes_per_line - BUFF_WIDTH*bytes_per_pixel;	\
    }
#define scrn_4hor_b(cpy,eq,bytes_per_pixel)			\
    for(y= BUFF_HEIGHT/2; y != 0; y--) {				\
								\
	/* left half */						\
	##cpy( scrn, tmp, BUFF_WIDTH/2);				\
	scrn += BUFF_WIDTH/2*bytes_per_pixel;			\
	tmp += BUFF_WIDTH/2;					\
								\
	/* right half */					\
	for(x=BUFF_WIDTH/2; x != 0; x--)	{			\
	    ##eq(scrn, *tmp);					\
	    scrn += bytes_per_pixel;				\
	    tmp --;						\
	}							\
	tmp -= BUFF_WIDTH;					\
	scrn += bytes_per_line - BUFF_WIDTH*bytes_per_pixel;	\
    }
int screen_4hor() {
    unsigned char * tmp, * scrn;
    int x,y;
	
    /* upper half of screen */	
    tmp = active_buffer + BUFF_SIZE/2;
    scrn = display_mem;
    SCRN(4hor_a)

    /* lower half of screen */
    tmp = active_buffer + BUFF_WIDTH*BUFF_HEIGHT - BUFF_WIDTH;
    scrn = display_mem + bytes_per_line*BUFF_HEIGHT/2;
    SCRN(4hor_b)

    return 0;
}


#define scrn_2verd(cpy,eq,bytes_per_pixel)		\
    for(y=BUFF_HEIGHT; y != 0; y--) {			\
	for(x=BUFF_WIDTH/2; x != 0; x--) {		\
	    ##eq(scrn, *tmp);				\
            scrn += bytes_per_pixel;			\
	    tmp += BUFF_WIDTH;				\
	}						\
	for(x=BUFF_WIDTH/2; x != 0; x--) {		\
	    ##eq(scrn, *tmp);				\
            scrn += bytes_per_pixel;			\
	    tmp -= BUFF_WIDTH;				\
	}						\
	tmp ++;						\
	scrn += bytes_per_line - bytes_per_pixel*BUFF_WIDTH;		\
    }

int screen_2verd() {
    if( BUFF_WIDTH/2 <= BUFF_HEIGHT) {
	int x,y;
	unsigned char * tmp = active_buffer;
	unsigned char * scrn = display_mem;
	SCRN(2verd)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}


#define scrn_r2verd(cpy,eq,bytes_per_pixel)			\
    for(y=BUFF_HEIGHT; y != 0; y--) {				\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn -= bytes_per_pixel;				\
	    tmp += BUFF_WIDTH;					\
	}							\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn -= bytes_per_pixel;				\
	    tmp -= BUFF_WIDTH;					\
	}							\
	tmp ++;							\
	scrn -= bytes_per_line - bytes_per_pixel*BUFF_WIDTH;	\
    }
int screen_r2verd() {
    if( BUFF_WIDTH/2 <= BUFF_HEIGHT) {
	int x,y;
	unsigned char * tmp = active_buffer;
	unsigned char * scrn = display_mem + 
	    bytes_per_line*(BUFF_HEIGHT-1) + bypp*(BUFF_WIDTH-1);
	SCRN(r2verd)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}


#define scrn_4kal_a(cpy,eq,bytes_per_pixel)			\
    for(y=BUFF_HEIGHT/2; y != 0; y--) {				\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn += bytes_per_pixel;				\
	    tmp += BUFF_WIDTH;					\
	}							\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn += bytes_per_pixel;				\
	    tmp -= BUFF_WIDTH;					\
	}							\
	tmp ++;							\
	scrn += bytes_per_line - bytes_per_pixel*BUFF_WIDTH;	\
    }
#define scrn_4kal_b(cpy,eq,bytes_per_pixel)			\
    for(y=BUFF_HEIGHT/2; y != 0; y--) {				\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn -= bytes_per_pixel;				\
	    tmp += BUFF_WIDTH;					\
	}							\
	for(x=BUFF_WIDTH/2; x != 0; x--) {			\
	    ##eq(scrn, *tmp);					\
	    scrn -= bytes_per_pixel;				\
	    tmp -= BUFF_WIDTH;					\
	}							\
	tmp ++;							\
	scrn -= bytes_per_line - bytes_per_pixel*BUFF_WIDTH;	\
    }
int screen_4kal() {
    if( BUFF_WIDTH/2 <= BUFF_HEIGHT) {
	unsigned char * tmp, * scrn;
	int x,y;
	
	tmp = active_buffer;
	scrn = display_mem;
	/* upper half */
	SCRN(4kal_a)

	tmp = active_buffer;
	scrn = display_mem + bytes_per_line * (BUFF_HEIGHT-1) + 
	    bypp*(BUFF_WIDTH-1);
	/* lower half */
	SCRN(4kal_b)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}


/*
 * big screen functions
 */
#define scrn_mirrorH(cpy,eq,bytes_per_pixel)			\
    for(i=BUFF_HEIGHT; i != 0; i--) {				\
	/* left part maps directly */				\
	##cpy(dst1, src, BUFF_WIDTH);				\
	    							\
	/* right part gets turned around */			\
	for(j=BUFF_WIDTH; j != 0; j--) {			\
	    ##eq(dst2, *src);					\
	    src ++;						\
	    dst2 -= bytes_per_pixel;				\
	}							\
	/* move pointer to next line */				\
	dst1 += bytes_per_line;					\
	dst2 += bytes_per_line + bytes_per_pixel*BUFF_WIDTH;	\
    }
int screen_mirrorH() {
    if( 2*BUFF_WIDTH <= disp_size.x) {
	int i,j;
	unsigned char * src = active_buffer;
	unsigned char * dst1 = display_mem;
	unsigned char * dst2 = display_mem + 2*bypp*BUFF_WIDTH - bypp;
	SCRN(mirrorH)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}


#define scrn_mirrorHV(cpy,eq,bytes_per_pixel)			\
    for(i=BUFF_HEIGHT; i != 0; i--) {				\
	/* upper left maps direct */				\
	##cpy(dst1, src, BUFF_WIDTH);				\
	/* lower left maps direct */				\
	##cpy(dst3, src, BUFF_WIDTH);				\
	/* upper & lower right gets turned around */		\
	for(j=BUFF_WIDTH; j != 0; j--) {			\
	    ##eq(dst2, *src);					\
	    ##eq(dst4, *src);					\
	    src ++;						\
	    dst2 -= bytes_per_pixel;				\
	    dst4 -= bytes_per_pixel;				\
	}							\
	/* move pointer to next line */				\
	dst1 +=  bytes_per_line;				\
	dst2 +=  bytes_per_line + bytes_per_pixel*BUFF_WIDTH;	\
	dst3 += -bytes_per_line;				\
	dst4 += -bytes_per_line + bytes_per_pixel*BUFF_WIDTH;	\
    }
int screen_mirrorHV() {
    if( (2*BUFF_WIDTH <= disp_size.x) && (2*BUFF_HEIGHT <= disp_size.y) ) {
	int i,j;
	unsigned char * src = active_buffer;
	unsigned char * dst1 = display_mem;
	unsigned char * dst2 = display_mem + 2*bypp*BUFF_WIDTH - bypp;
	unsigned char * dst3 = display_mem + 2*BUFF_HEIGHT*bytes_per_line 
	    - bytes_per_line;
	unsigned char * dst4 = display_mem + 2*BUFF_HEIGHT*bytes_per_line 
	    + 2*bypp*BUFF_WIDTH - bypp - bytes_per_line;
	SCRN(mirrorHV)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}



#define scrn_mirrorV(cpy,eq,bytes_per_pixel)			\
    for(i=BUFF_HEIGHT; i != 0; i--) {				\
	/* do 1 line */						\
	##cpy(dst1, src, BUFF_WIDTH);				\
	##cpy(dst2, src, BUFF_WIDTH);				\
	/* move pointer to next line */				\
	src += BUFF_WIDTH;					\
	dst1 += bytes_per_line;					\
	dst2 -= bytes_per_line;					\
    }
int screen_mirrorV() {
    if( 2*BUFF_HEIGHT <= disp_size.y ) {
	int i;
	unsigned char * src = active_buffer;
	unsigned char * dst1 = display_mem;
	unsigned char * dst2 = display_mem + 2*BUFF_HEIGHT*bytes_per_line 
	    - bytes_per_line;
	SCRN(mirrorV)
    } else {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }
    return 0;
}


#define scrn_sheet(cpy,eq,bytes_per_pixel)				\
   for(i=BUFF_HEIGHT; i != 0; i--) {					\
       xy t = p;							\
       for(j=BUFF_WIDTH; j != 0; j--) {					\
	   unsigned char * d = dst +					\
	       ((p.y + height_offset[*src].y) >> 16) * bytes_per_line +	\
	       ((p.x + height_offset[*src].x) >> 16) * bytes_per_pixel;	\
	   ##eq(d, *src | 128);						\
	   src++;							\
	   p.x += s1.x; p.y += s1.y;					\
       }								\
       p.x = t.x + s2.x;						\
       p.y = t.y + s2.y;						\
   }
static float rot[3] = {0,0,0};
static float P[4][3] = { {-1,-1,0}, {1,-1,0}, {1,1,0}, {0,0,1} };
void rotate(float p[3], float rot[3], float r[3]) {
    float t1[3], t2[3];
    /* first around x axis */
    t1[0] = p[0];
    t1[1] = p[1] * cos(rot[0]) + p[2] * sin(rot[0]);
    t1[2] = -p[1] * sin(rot[0]) + p[2] * cos(rot[0]);

    /* around y axis */
    t2[0] = t1[0] * cos(rot[1]) + t1[2] * sin(rot[1]);
    t2[1] = t1[1];
    t2[2] = -t1[0] * sin(rot[1]) + t1[2] * cos(rot[1]);

    /* around z axis */
    r[0] = t2[0] * cos(rot[2]) + t2[1] * sin(rot[2]);
    r[1] = -t2[0] * sin(rot[2]) + t2[1] * cos(rot[2]);
    r[2] = t2[2];
}
float sqr(float a) {
    return a*a;
}
int screen_sheet() {
    float x,y,z,l;
    unsigned char * src = active_buffer;
    unsigned char * dst = display_mem + 
	BUFF_WIDTH*bypp + bytes_per_line*BUFF_HEIGHT;
    unsigned char * clr = display_mem;
    int i,j;
    float ro[4][3];
    xy height_offset[256];
    xy p;
    xy s1, s2;
    const float SC = 1 << 16;
    double scale;
    
    if( (2*BUFF_WIDTH > disp_size.x) || (2*BUFF_HEIGHT > disp_size.y) ) {
	change_update_screen(CHANGE_NEXT);
	return 1;
    }

    /* calculate the maximal size of the "sheet" */
    x = sqr(BUFF_WIDTH/2);
    y = sqr(BUFF_HEIGHT/2);
    z = sqr((float)(BUFF_WIDTH)/640.0 * 256.0);
    l = sqrt(x + y + z);

    /* make sure, the image always fits into the display region */
    scale = (float)min(BUFF_WIDTH, BUFF_HEIGHT) / l;
    
    /* rotate a little bit */
    rot[0] += 0.006;
    rot[1] += 0.010;
    rot[2] += 0.004;
    for(i=0; i < 4; i++)
	rotate(P[i], rot, ro[i]);
	
    for(i=0; i < 256; i++) {
	height_offset[i].x = ro[3][0] * i * (float)BUFF_WIDTH / 640.0 * SC;
	height_offset[i].y = ro[3][1] * i * (float)BUFF_WIDTH / 640.0 * SC;
    }

    /* calculate the step sizes */
    s1.x = (ro[1][0] - ro[0][0]) * scale * SC/2.0;
    s1.y = (ro[1][1] - ro[0][1]) * scale * SC/2.0;

    s2.x = (ro[2][0] - ro[1][0]) * scale * SC/2.0;
    s2.y = (ro[2][1] - ro[1][1]) * scale * SC/2.0;

    /* starting point */
    p.x = -BUFF_WIDTH/2 * s1.x -BUFF_HEIGHT/2 * s2.x;
    p.y = -BUFF_WIDTH/2 * s1.y -BUFF_HEIGHT/2 * s2.y;

    /* clear screen */
    for(i=2*BUFF_HEIGHT; i != 0; i--) {
	bzero(clr, 2*BUFF_WIDTH*bypp);
	clr += bytes_per_line;
    }
    
    /* now really draw */
    SCRN(sheet)

    return 0;
}





