#include "cthugha.h"
#include "display.h"
#include "options.h"
#include "interface.h"
#include "disp-sys.h"

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "initial_palettes.c"

int change_palette_imm(int);
feature palettes = {
    "palette",
    NULL,
    0,
    CHANGE_RANDOM,
    change_palette_imm
};
palette * active_palette = NULL;
char palette_first[256] = "";			/* Start with this palette */

int display_internal_pal = 1;			/* use internal pal. */
int display_external_pal = 1;			/* use external pal. */

unsigned int bitmap_colors[256];		/* "compiled" palette */

int read_palette(FILE * file, char * name);
char * palette_path[] = {
    "./",
    "./map/",
    INST_LIB "map/",
    ""	
};

int colormapped=1;				/* 0 .. True/Direct color
						   1 .. All 256 color cells
						   2 .. Only some cells */

/* when using only some colorcells (e.g. when drawing on the root window)
   update to the palette are done immediately without smoothing.
   */

int load_palettes() {
    int i;
	
    /* Builtin palettes */
    if ( display_internal_pal ) {
	feature_entry new_pal;
	palette * pal;

	printfv(2,"  preparing internal palettes...\n");

	new_pal.desc[0] = '\0';
	new_pal.use = 1;
	    
	for(i=0; i < sizeof(initial_palettes)/sizeof(palette); i++) {
	    new_pal.data = initial_palettes[i];
	    sprintf( new_pal.name, "Internal_%d", i);

	    add_feature(&palettes, &new_pal); 
	}

	/* create one general palette */
	new_pal.data = cth_memory(NULL, sizeof(palette), 
		"Can not allocate memory for internal palette.\n");
	pal = (palette*)new_pal.data;
	for(i=0; i < 64; i++) {
	    (*pal)[i][0]	= i << 2;
	    (*pal)[i][1]	= 0;
	    (*pal)[i][2]	= 0;

	    (*pal)[i+64][0] 	= 0;
	    (*pal)[i+64][1]	= i << 2;
	    (*pal)[i+64][2] 	= 0;

	    (*pal)[i+128][0] 	= 0;
	    (*pal)[i+128][1]	= 0;
	    (*pal)[i+128][2] 	= i << 2;

	    (*pal)[i+192][0] 	= i << 2;
	    (*pal)[i+192][1]	= i << 2;
	    (*pal)[i+192][2] 	= i << 2;
	}
	sprintf(new_pal.name, "Internal_%d", palettes.nr_entries);

	add_feature(&palettes, &new_pal);
    }
	
    /* read palettes from file */
    if ( display_external_pal) {
	printfv(2,"  loading external palettes...\n");
	load(palette_path, "/map/", ".map", read_palette);
	printfv(2,"  number of loaded palettes: %d\n", palettes.nr_entries);
    }
    return 0;
}

/*
 * store the palette with 8 bytes per color
 */
int read_palette(FILE * file, char * name) {
    char dummy[256];
    int i, j, r,g,b;
    feature_entry new_pal;
    palette * pal;

    if ( feature_defined(name, &palettes) && !double_load) {
	printfv(3," ... already loaded\n");
	return 1;
    }
    new_pal.data = cth_memory(NULL, sizeof(palette), 
		     "Can not allocate memory for external palette\n");
    pal = (palette*)new_pal.data;

    for(i=0; i < 256; i++) { 
	if ( fscanf( file, "%d %d %d", &r, &g, &b) < 3) {
	    printfv(3,"\n    Can't read at line: %d (%s)", i,name);
	    if ( i == 0) {				/* nothing read */
		printfv(3," ... skipping file");
		free(new_pal.data);
		return 1;
	    }
	    printfv(3," ... filling with black");
	    for(; i < 256; i++)				/* fill with black */
		for(j=0;j<3;j++)
		    (*pal)[i][j] = 0;
	    break;
	}
	fgets( dummy, 255, file);
	(*pal)[i][0] = r;
	(*pal)[i][1] = g;
	(*pal)[i][2] = b;
    }

    strncpy(new_pal.name, name, MAX_NAME_LEN);
    new_pal.desc[0] = '\0';
    new_pal.use = 1;

    add_feature(&palettes, &new_pal);

    return 0;
}

/*
 * Set the first palette
 *   - allocate palette
 */
int init_palettes() {
    if( init_palettes_sys() )
	return 1;
    update_palette();
    return 0;
}

int exit_palettes() {
    exit_palettes_sys();
    return 0;
}

/*
 * Functions to change/selct the palette
 *
 * changes (smooth_palette) by "Stanislav V. Voronyi" <stas@use.kharkov.ua>
 */
palette cur_palette;
static palette new_palette;
static int pal_changed = 0;
static int pal_text = 0;	/* a "text-palette" is on screen */


void cth_setpalette(palette pal, int immed) {
    static int cur_empty = 1;

    memcpy( new_palette, pal, sizeof(palette) );	/* save new palette */

    if ( display_text_on_screen) {			/* text-mode */
	int i,j;
	/* create a dimmed palette */

	/* first 255 entries are darkend by a factor of 2 */
	for(i=0; i < 255; i++)
	    for(j=0; j < 3; j++)
		new_palette[i][j] >>= 1;

	/* set last entry to a bright color */		
	for(j=0; j < 3; j++)
	    cur_palette[255][j] = new_palette[255][j] = display_text_color[j];

	/* don't use color index 255, when text is on the screen */
	if( draw_mode == DM_direct) {
	    draw_mode = DM_tmp_mapped;

	    for(i=0; i < 255; i++)
		bitmap_colors[i] = i;
	    bitmap_colors[255] = 254;
	}
	    
	pal_text = 1;
    } else {
	pal_text = 0;
	if(draw_mode == DM_tmp_mapped)
	    draw_mode = DM_direct;
    }

    pal_changed = 1;				/* palette entries changed */

    if( cur_empty || immed || (colormapped == 2) ) {
 	memcpy( cur_palette, new_palette, sizeof(palette) );
	cur_empty=0;

	smooth_setpalette();			/* really set the palette */
    }
}

/*
 * brings the current palette nearer to the desired palette (new_palette)
 * each entry is changed by at most 4.
 */
void smooth_setpalette(void) {
    int i,j;

    if (display_text_on_screen && !pal_text) {
	update_palette();
	return;
    }

    if (!display_text_on_screen && pal_text) {
	update_palette();
	return;
    }

    if (! pal_changed )				/* no update needed */
	return;

    /* count if anything changed */
    pal_changed = 256*3;
    for(i=0; i < 256; i++)
	for(j=0; j < 3; j++) {
	    int d = new_palette[i][j] - cur_palette[i][j];
	    d = min( max(d, -4), 4);	/* d between -4 and +4 */
	    if( d != 0)
		cur_palette[i][j] += d;
	    else
		pal_changed --;
	}

    /* really set the palette now */
    set_palette_sys();
}

/*
 * set a new palette
 */
int change_palette(int to, int immed) {
    active_palette = feature_change(to, &palettes)->data;
    cth_setpalette( *active_palette, immed);
    return 0;
}
int change_palette_imm(int to) {
    return change_palette(to, 1);
}
int select_palette() {
    selection(&palettes, "Select Palette\n", change_palette_imm);
    return 0;
}
int update_palette() {
    return change_palette(palettes.current,0);
}












