/*
 * some important parts written by:
 *  Mark Vojkovich (mvojkovi@ucsd.edu) 
 * and
 *  modified to allow status to change without reseting the counter...  clp 
 */

#include "cthugha.h"
#include "display.h"
#include "disp-sys.h"

#include <string.h>
#include <stdlib.h>

int display_text_on_screen = 0;		/* text is currently on screen */
int display_text_time = 300;
int display_text_color[3] = {0, 255, 255};

typedef struct {
    enum {
	left   = 'l', 
	right  = 'r', 
	center = 'c'
    } justification;
    int remove;				/* remove after some time */
    int time;				/* when was it put on screen */
    char text[256];
} text_line;

text_line text_lines[MAX_TEXT_LINES];


/* 
 * Display a text on the screen 
 * The text is not displayed immediately, only with the next
 * refresh of the screen.
 */

/*
 * clear the text display 
 */
int display_print_clear() {
    int i;

    for(i=0; i < MAX_TEXT_LINES; i++)
	text_lines[i].text[0] = '\0';
    if ( display_text_on_screen) {		/* something on display */
	display_text_on_screen = 0;
	display_clear = 1;			/* need a full redraw now */
    }
    return 0;
}

/*
 * print some text to screen
 */
int display_print(char * text, int line, int justification, int remove) {
    char * ntext; 

    if ( text == NULL) {			/* deactivate display */
	return display_print_clear();
    } 

    if(line == -1)				/* random */
	line = rand() % (disp_size.y - 8)/8;

    if(line == -2) {				/* at end, scroll */
	/* search last displayed line */
	for(line = MAX_TEXT_LINES -1; 
	    (text_lines[line].text[0] == '\0') && (line >= 0);
	    line --);
	line ++;
	if(line >= (disp_size.y/8)) {		/* need scrolling */
	    int i;
	    for(i=0; i < line-1; i++)
		memcpy(text_lines+i, text_lines+i+1, sizeof(text_line));
	    line --;
	}
    }

    ntext = strchr(text,'\n');
    while( ntext != NULL) {
	int len = min(ntext-text, 254);
	strncpy(text_lines[line].text, text, len);
	text_lines[line].text[len] = '\0';
	text_lines[line].justification = justification;
	text_lines[line].remove = remove;
	text_lines[line].time = gettime();
	line ++;
	if ( line >= disp_size.y/8)
	    line = 0;
	text = ntext+1;
	ntext = strchr(text, '\n');
    }
    strncpy(text_lines[line].text, text, 256);
    text_lines[line].justification = justification;
    text_lines[line].remove = remove;
    text_lines[line].time = gettime();

    display_clear = 1;				/* need a full redraw now */
    display_text_on_screen = 1;

    return 0;
}




/*
 * bring the current active text to the screen
 */
int show_text() {
    int line;

    if ( ! display_text_on_screen ) 
	return 0;
		
    /* write text to screen/buffer line by line */
    for(line = 0; line < MAX_TEXT_LINES; line++) {
	int len = min(strlen(text_lines[line].text), disp_size.x/8);
	int x;

	/* display only, if there is something to dispaly */
	if( text_lines[line].text[0] == '\0')
	    continue;

	/* check if it is time to remove this line */
	if( text_lines[line].remove ) {
	    if( (gettime() - text_lines[line].time) > display_text_time)
		text_lines[line].text[0] = '\0';
	}

	switch( text_lines[line].justification) {
	case left:
	    x = 0;
	    break;
	case center:
	    x = 8 * (disp_size.x/8 - len) / 2;
	    break;
	case right:
	    x = 8 * (disp_size.x/8 - len);
	    break;
	default:
	    x = 0;
	}
	    
	display_print_string_sys(x, 8*line, text_lines[line].text);
    }
    return 0;
}


