#include "cthugha.h"
#include "feature.h"
#include "display.h"
#include "sound.h"
#include "translate.h"
#include "action.h"

#include <stdio.h>
#include <string.h>


feature_entry feature_null = {NULL, 0, "none", ""};

int add_feature(feature * features, const feature_entry * entry) {
    features->entries = 
	cth_memory(features->entries,
		   (features->nr_entries+1) * sizeof(feature_entry),
		   "Can not get memory for feature entry.\n");

    memcpy(features->entries + features->nr_entries,  entry,
	   sizeof(feature_entry));

    features->nr_entries ++;
    return 0;
}


/*
 * Return a pointer to the new feature entry
 *
 * also sets the features->current entry.
 */
feature_entry * feature_change(int to, feature * features) {
    int i;
    int nouse;

    /* check for empty set */
    if ( features->nr_entries == 0)
	return &feature_null;
	
    /* check, of there is any featureion we can use */
    nouse = 1;
    for(i=0; i < features->nr_entries; i++)
	if ( features->entries[i].use ) {
	    i = features->nr_entries;
	    nouse = 0;
	}

    /* if non in use, return first */
    if ( nouse)
	return features->entries;
	
    switch(to) {
    case CHANGE_RANDOM:
	do {
	    to = rand() % features->nr_entries;
	} while( ! features->entries[to].use );
	break;
    case CHANGE_NEXT:
	to = features->current;
	do {
	    to = (to + 1) % features->nr_entries;
	} while(features->entries[to].use == 0);
	break;
    case CHANGE_PREV:
	to = features->current;
	do {
	    to = (to > 0) ? to -1 : features->nr_entries - 1;
	} while(features->entries[to].use == 0);
	break;
    default:
	if ( to < 0)
	    to = 0;
	if ( to >= features->nr_entries) 
	    to = features->nr_entries -1;
	while( features->entries[to].use == 0) 
	    to = (to+1) % features->nr_entries;
    }
    features->current = to;
    return features->entries + to;
}

/*
 * change featureion, allowing names
 */
feature_entry * feature_change2(char * to, feature * features) {
    int nr;
    char * pos;

    /* do nothing if to is empty */
    if ( *to == '\0' ) {
	if ( features->current == CHANGE_RANDOM )
	    return feature_change(CHANGE_RANDOM, features);
	else
	    return features->entries + features->current;
    }
    /* first try to read as number */
    nr = strtol(to, &pos, 0);
    if ( pos == to ) {
	/* not a number, so try as name */
	for(nr = 0; nr < features->nr_entries; nr ++)
	    if(strcasecmp(to, features->entries[nr].name) == 0) {
		return feature_change(nr, features);
	    }
	nr = CHANGE_RANDOM;
    }

    return feature_change(nr, features);
}

/*
 * check, if a given name is in the entries for feature
 */
int feature_defined(const char * name, const feature * features) {
    int i;
    for( i=0; i < features->nr_entries; i++)
	if ( strcmp(features->entries[i].name, name) == 0)
	    return 1;
    return 0;
}

/****************************************************************************/

/*
 * change a numerical value
 */
int val_change(int to, int min, int max, int old) {
	
    switch(to) {
    case CHANGE_RANDOM:
	if ( max < min)
	    return min;
	return rand() % (1+max-min) + min;
    case CHANGE_NEXT:
	return ( old >= max) ? min: old + 1; 
    case CHANGE_PREV:
	return ( old <= 0) ? max : old - 1;
    default:
	if ( to < min)		to = min;
	if ( to > max)		to = max;
	return to;
    }
}



/*
 * Set first options in use
 */
int init_startup() {

    /* set palette */
    active_palette = feature_change2(palette_first, &palettes)->data;

    /* set screen-function */
    update_screen = feature_change2(screen_first, &screens)->data;

    /* use first pcx if specified */
    if( pcx_first[0] != '\0' ) { 
        /* select first pcx-file */
        active_pcx = feature_change2(pcx_first, &pcxs)->data;
        if ( active_pcx != NULL) {
	    show_pcx(1,0);		/* display the select pcx to active*/
	    memcpy(passive_buffer, active_buffer, BUFF_SIZE); 
	    active_palette =		/* and set corresponding palette */
  	        feature_change(pcx_palettes[pcxs.current],
			       &palettes)->data;
	}
    } else {
    	change_pcx(CHANGE_NEXT);
    }


    /* select first wave */
    display_wave = feature_change2(wave_first, &waves)->data;

    /* set first translation */
    active_translation = feature_change2(translate_first, &translations)->data;

    /* set first flame-function */
    flame = feature_change2(flame_first, &flames)->data;

    /* set first table */
    change_table(table_first);

    /* set first massage */
    change_massage_style(massage_first);

    return 0;
}

/*
 * save startup-section if --save was given
 */
int exit_startup() {
    if ( options_save) {
	write_ini();
	save_buffer();
    }
    return 0;
}
