#include "cthugha.h"
#include "information.h"
#include "options.h"
#include "display.h"
#include "sound.h"
#include "action.h"
#include "translate.h"
#include "keys.h"

#include <unistd.h>
#include <stdio.h>

char * info_credits[] = {
    "--------------------------------------------------------------------------------",
    "C T H U G H A - L  0.9",
    "--------------------------------------------------------------------------------", 
    "by Harald Deischinger                  ",
    "   Am Edhuegel 46                      ",
    "   4115  Kleinzell                     ",
    "   AUSTRIA                             ",
    "email: k3096e5@c210.edvz.uni-linz.ac.at",
    "--------------------------------------------------------------------------------",
    " ",
    "Original Programm (CTHUGHA V5.1, V5.3) by:",
    "  Coded by - Torps Productions:           ",
    "    The Digital Aasvogel Group - 1995     ", 
    "  Original Idea & Code:                   ",
    "    Kevin Burfitt (zaph@torps.apana.org.au)", 
    " ", 
    "--------------------------------------------------------------------------------",
    " ",
    "Cthugha WWW-Page",
    "----------------",
    "http://www.afn.org/~cthugha",
    " ",
    "Cthugha newsgroup",
    "-----------------",
    "alt.graphics.cthugha",
    " ",
    "--------------------------------------------------------------------------------",
    " ",
    "Want to register?",
    " ",
    "for personal use",
    "----------------",
    "send a Postcard to me",
    "or a CD if you have too much money",
    " ",
    "for commercial use",
    "------------------",
    "register for US$50 or 2 CDs",
    " ",
    "for magazines",
    "-------------",
    "please send a copy of your review",
    "or coverdisk to me",
    " ",
    " ",
    "-------------------------------------------------------------------------------", 
    " ",
    "The first postcard I received !", 
    "-------------------------------",
    " ",
    "  Luigi - Milano, Italy     - special thanks",
    " ",
    "The second (electronic) postcard I received !",
    "---------------------------------------------",
    " ",
    "  Kevin McCarthy - USA",
    " ",
    "The next three postcards I received !",
    "-------------------------------------",
    " ",
    "  Reinhold Platzer - Germany",
    "  Danke - Thanks - Merci",
    " ",
    "Some more responses I got !",
    "---------------------------",
    "Fizz",
    "Wolfgang Surholt",
    "Jeremiah Cox",
    "The Mangement - Lick Observatory",
    "Christian Lorenz",
    "The Prince of Darkness",
    "   ",
    " ",
    " ",
    "Please send me some more",
    " ",
    " ",
    "-------------------------------------------------------------------------------", 
    " ",
    "These guys are the main authors of Cthugha",
    "------------------------------------------",
    " ",
    "  Kevin Burfitt   - original Author                   ",
    "                    DOS and Windows 95                ",
    "  Rus Maxham      - Mac Version                       ",
    "  Mark Vojkovich  - DGA support                       ",
    "  Jeff The Riffer - maintains WWW page and FTP archive",
    " ",
    "Some more guys who helped to improve Cthugha-L !", 
    "------------------------------------------------",
    " ",
    "Christopher L. Platt              ",
    "Stanislav V. Voronyi              ",
    "Torsten Martinsen                 ",
    "Jarkko Lietolahti                 ",
    "Jan Kujawa                        ",
    "Someone who gave only root as name",
    " ",
    " ",
    " ",
    " ",
    " "
};
int info_nr_credits = sizeof(info_credits) / sizeof(char*);


/*****************************************************************************/
		
/* Modifications to update status "interactively" by 
 * Christopher L. Platt  (croutons%crunchy.uucp@dartmouth.edu)
 */

int do_status = 0;

 /* Generate status-information-string
 */
void status(int reset) {
    char s[2000];
    float fps;
    int i;

    display_print_clear();

    /* toggle info */
    if ( do_status && reset ) {
	do_status = 0;
	return;
    }

    i = time(0) - display_start;
    fps = (i==0) ? 0: ((float)display_frames)/((float)(i));

    sprintf(s, 
	    "Wave    : %s (%s)\n"
	    "Flame   : %s (%s)\n"
	    "Display : %s (%s)\n"
	    "Palette : %s (%s)\n"
	    "Transl. : %s (%s)\n"
	    "Next PCX: %s (%s)\n"
	    "Table   : %d\n"
	    "Massage : %d\n"
	    "Channels: %s\n"
	    "Rate    : %d\n"
	    "%s%s%s\n"
	    "Frames/s: %2.2f",

	    waves.entries[waves.current].name,
	    waves.entries[waves.current].desc,

	    flames.entries[flames.current].name,
	    flames.entries[flames.current].desc,

	    screens.entries[screens.current].name,
	    screens.entries[screens.current].desc,

	    palettes.entries[palettes.current].name,
	    palettes.entries[palettes.current].desc,

	    translations.entries[max(translations.current,0)].name,
	    translations.entries[max(translations.current,0)].desc,

	    pcxs.entries[max(pcxs.current,0)].name,
	    pcxs.entries[max(pcxs.current,0)].desc,

	    active_table,
	    sound_massage_style,

	    (sound_stereo == 2) ? "Stereo" : "Mono",
	    sound_sample_rate,
	    action_lock ? "LOCKED " : "",
	    (sound_FFT) ? "FFT " : "",
	    (sound_flashlight) ? "Flash" : "",
	    fps);
		
    display_print(s, 0, 'l', reset);
    do_status = 1;
}		

void update_status() {
    if ( do_status )
	status(0);
}

void clear_status() {
    do_status = 0;
}

int status_active() {
    return do_status;
}

/*****************************************************************************/

/* 
 * display help-screen
 */
void help() {

    display_print( "f/F  Chg Flame     w/W  Chg Wave         \n"
		   "d/D  Chg Display   b/B  Chg Table        \n"
		   "t/T  Chg Transl.   m/M  Chg Snd-massage  \n"
		   "p/P  Chg Palette   s    Toggle flashlight\n"
		   "e    Toggle FFT                          \n"
		   "l    Lock          i    Information      \n"
		   "x/X  show PCX      +/- Sample-Rate       \n"
		   "*    mono/stereo   </> Minnoise          \n"
		   "F1/? This Help     F2  Select Display    \n"
		   "F3   Select Wave   F4  Select Flame      \n"
		   "F5   Select Trans. F6  Select Palette    \n"
		   "F7   Select PCX    g   Chg. gen. flame   \n"
		   "o    Optioms       c   Sound/CD Control  \n"   
		   "a    write ini-file with current sel.    \n"
		   "n    reconnect to cthugha-server         \n"
		   "q/ESC  Exit        #   Print screen      \n",
		   0,
		   'l', 1);
}

