#include "cthugha.h"
#include "cd_player.h"
#include "sound.h"
#include "interface.h"
#include "keys.h"
#include "display.h"

/* 
 *  CD-Player-user-interface (and mixer)
 *  This is much more than a simple CD-Player. It's more a control-panel
 *  for sound-related stuff.
 */
int cd_player() {

    char str[512];
    char str2[1024];
    char vstr[16];
    int key = CK_NONE;
    static int sel = 0;
    int nr_sel;
    int skip=0;
    int fast=0;
    int done=0;
    int i;

    char * voltostr(int volume, char * str) {
	if ( volume < 0) {
	    strcpy( str, "default");
	    return str;
	}
	sprintf(str, "%d", volume & 255);
	return str;
    }

    if ( subchnl.cdsc_audiostatus == CDROM_AUDIO_PLAY )
	sprintf(str, 
		"CD-Player Track: %d %d:%02d (%d - %d) %d:%02d",
		subchnl.cdsc_trk,
		subchnl.cdsc_reladdr.msf.minute,
		subchnl.cdsc_reladdr.msf.second,
		first, last,
		subchnl.cdsc_absaddr.msf.minute,
		subchnl.cdsc_absaddr.msf.second);
    else if ( subchnl.cdsc_audiostatus == CDROM_AUDIO_PAUSED )
	sprintf(str, 
		"CD-Player Track: %d %d:%02d (%d - %d) %d:%02d pause",
		subchnl.cdsc_trk,
		subchnl.cdsc_reladdr.msf.minute,
		subchnl.cdsc_reladdr.msf.second,
		first, last,
		subchnl.cdsc_absaddr.msf.minute,
		subchnl.cdsc_absaddr.msf.second);
    else
	if(ejected)
	    sprintf(str,"CD-Player: ejected");
	else if (no_disk)
	    sprintf(str,"CD-Player: no disk in drive");
	else
	    sprintf(str,"CD-Player Track: stopped");
    
    sprintf(str2,
	    "%s\n"
	    "<- prev(7)  pause(8)  -> next(9) \n"
	    "<< rev(4)   stop(5)   >> fwd(6)  \n"
	    "            %s             \n"
	    "\n"
	    "%cAutoplay:       %10s%c\n"
	    "%cPlayloop:       %10s%c\n"
	    "%cRandomplay:     %10s%c\n"
	    "%cEject On End:   %10s%c\n"
	    "\n"
	    "%cSample rate:    %10d%c\n"
	    "%cChannels:       %10s%c\n",
	    str, ejected ? "close(2)":"eject(2)",
	    
	    (sel == 0) ? '>' : ' ',
	    (cd_first_track == -1) ? "off" : "on",
	    (sel == 0) ? '<' : ' ',
	    
	    (sel == 1) ? '>' : ' ',
	    (cd_loop) ? "on" : "off",
	    (sel == 1) ? '<' : ' ',
	    
	    (sel == 2) ? '>' : ' ',
	    (cd_randomplay) ? "on" : "off",
	    (sel == 2) ? '<' : ' ',
	    
	    (sel == 3) ? '>' : ' ',
	    (cd_eject_on_end) ? "on" : "off",
	    (sel == 3) ? '<' : ' ',
	    
	    (sel == 4) ? '>' : ' ',
	    sound_sample_rate,
	    (sel == 4) ? '<' : ' ',
	    
	    (sel == 5) ? '>' : ' ',
	    (sound_stereo == 1) ? "mono" : "stereo",
	    (sel == 5) ? '<' : ' '
	);
    nr_sel = 6;
    for(i=0; i < mixer_nr_names; i++) {
	if(mixer_names[i]) {
	    sprintf(str2, 
		    "%s\n"
		    "%cVolume %7s: %10s%c",
		    str2,
		    (sel == nr_sel) ? '>' : ' ',
		    mixer_names[i],
		    voltostr( mixer_volumes[i], vstr),
		    (sel == nr_sel) ? '<' : ' ');
	    nr_sel ++;
	}
    }
    
    display_print(str2, 0, 'l', 0);

    while( !done && ((key = getkey()) != CK_NONE)) {

	switch( key ) {
	    /* CD-Player keys */
	case '9':
	    skip ++;
	    break;
	case '7':
	    skip --;
	    break;
	case '6':
	    fast += 15;
	    break;
	case '4':
	    fast -= 15;
	    break;
	case '8':
	    cd_pause();
	    break;
	case '5':
	    cd_stop();
	    break;
	case '2':
	    cd_eject();
	    break;
	
	    /* cursor movement up and down */
	case CK_UP:
	    sel = (sel - 1 + nr_sel) % nr_sel;
	    break;
	case CK_DOWN:
	    sel = (sel + 1) % nr_sel;
	    break;

	case '+': case CK_RIGHT:
	    switch( sel) {
		/* change CD-Player settings */
	    case 0:
		cd_first_track = 1;
		break;
	    case 1:
		cd_loop = 1;
		cd_eject_on_end = 0;
		break;
	    case 2:
		cd_randomplay = 1;
		break;
	    case 3:
		cd_eject_on_end = 1;
		cd_loop = 0;
		break;
		/* increase dsp settings */
	    case 4:
		sound_sample_rate += 1000;
		sound_dsp_update();
		break;
	    case 5:
		sound_stereo = (sound_stereo == 1) ? 2 : 1;
		sound_dsp_update();
		break;
		/* increase mixer settings */
	    default:
		mixer_volumes[sel-6] < 0 ? 
		    (mixer_volumes[sel-6] = 25700):
			(mixer_volumes[sel-6] += 257);
		sound_set_mixer();
		break;
	    }
	    break;
	case '-': case CK_LEFT:
	    switch( sel) {
		/* change CD-Player settings */
	    case 0:
		cd_first_track = -1;
		break;
	    case 1:
		cd_loop = 0;
		break;
	    case 2:
		cd_randomplay = 0;
		if ( random_played) {
		    free(random_played);
		    random_played = NULL;
		}
		cd_track(subchnl.cdsc_trk);
		break;
	    case 3:
		cd_eject_on_end = 0;
		break;
		/* decrease dsp settings */
	    case 4:
		if ( (sound_sample_rate -= 1000) <= 0)
		    sound_sample_rate = 0;
		sound_dsp_update();
		break;
	    case 5:
		sound_stereo = (sound_stereo == 1) ? 2 : 1;
		sound_dsp_update();
		break;
		/* decrease mixer settings */
	    default:
		if ( (mixer_volumes[sel-6] -= 257) < 0) {
		    mixer_volumes[sel-6] = 0;
		}
		sound_set_mixer();
		break;
	    }
	    break;
	default:		/* unhandled key */
	    done = 1;
	}
    }
    if ( skip != 0) {
	cd_next(skip);
    }
    if ( fast != 0) {
	cd_fast(fast);
    }

    return key;
}
