#include "cthugha.h"
#include "options.h"
#include "action.h"

#include <stdio.h>
#include <dirent.h>
#include <string.h>

int double_load = 0;			/* allow double loading */
int options_save = 0;			/* save options (and buffer) at end */

/*****************************************************************************/
void load_quiet_strings(char * fname) {

    FILE * file;
    char * s;
	
    if ( (fname == NULL) || (*fname == '\0'))
	return;
	
    if( (file = fopen(fname, "r")) == NULL) {
	printfee("Can't open quiet-strings-file %s", fname);
	return;
    }

    nr_silence_strings = 0;		
    do {
	/* read a line */
	s = fgets(silence_strings[nr_silence_strings], 255, file);	
	if ( s != NULL)
	    nr_silence_strings ++;
    } while ( (nr_silence_strings < MAX_SILENCE_STRINGS) && (s != NULL) );

    /* check if file was empty */
    if ( nr_silence_strings == 0) {
	nr_silence_strings = 1;
	strcpy( silence_strings[0], "Where is the musc?");
    }
}

/*****************************************************************************/

/*
 * general load-function
 */
int load(char * search_path[], char * extra_path, char * extension, 
	 int (* load_function)(FILE *, char *)) {
    int path;
    DIR * directory;
    FILE * file;	
    struct dirent * entry;
    char total_name[255];
    char extra_path_dir[512];

    int has_extension() {
	char * pos = strstr( entry->d_name, extension);
	if ( pos == NULL)
	    return 0;
	if ( (pos[strlen(extension)]=='\0') || (pos[strlen(extension)]=='.') ) 
	    return 1;
	return 0;
    }
    int is_compressed() {
	int l = strlen( entry->d_name);
	if ( l < 3)
	    return 0;
	if ((entry->d_name[l-3] == '.') && 
	    (entry->d_name[l-2] == 'g') && 
	    (entry->d_name[l-1] == 'z') )
	    return 1;
	return 0;
    }
    void load_dir(char * dir) {
	int compressed;

	if ( (directory = opendir( dir )) != NULL) {
	    while( (entry = readdir(directory))	 != NULL) {
		if ( ! has_extension() ) 
		    continue;

		/* create real filename */
		strncpy( total_name, dir, 255);
		strncat( total_name, entry->d_name, 255);

		compressed = is_compressed();

		printfv(3, "    ");

		if ( compressed ) {	  
		    /* open with 'gzip' - as pipe */
		    char cmd[512];

		    printfv(3,"uncompressing and ");

		    sprintf(cmd, "gzip -cd %s", total_name);

		    file = popen( cmd, "r");		
		    
		} else {
		    /* normal open - as file */
		    file = fopen( total_name, "r");
		}

		printfv(3,"loading: %s", total_name);
		
		/* now do the loading */
		if ( file != NULL)  {
		    /* remove rest of name */
		    *strstr(entry->d_name, extension) = '\0';
		    /* file was openen successfully  - now read it */
		    if ( (*load_function)( file, entry->d_name) == 0)
			printfv(3," ... OK\n");

		    if ( compressed) 
			pclose(file);	/* close pipe */
		    else
			fclose(file);	/* close file */

		} else {		/* file/pipe could not be opened */
		    printfv(3," ... x\n");
		}
	    }
	    closedir(directory);
	}
    }
	    
    /* load normal search path */
    path = 0;
    while( search_path[path][0] != '\0') {
	load_dir( search_path[path] );
	path ++;
    }
    /* load from extra path */
    if ( extra_lib_path[0] != '\0') {
	strcpy( extra_path_dir, extra_lib_path);
	strncat( extra_path_dir, extra_path, 512);
	load_dir( extra_path_dir);
    }
    
    return 0;
}

