#include "cthugha.h"
#include "sound.h"

#include <sys/soundcard.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

int sound_sample_rate = 44000;			/* sampel rate */
int sound_sync = 0;				/* reset sound after reading */
int sound_stereo = 2;
int sound_bps = 8;				/* bits per sample */
int sound_fragments = 0;			/* how many fragments to use */

static int sound_handle;			/* descriptor to dsp_file */
static int sound_fmt = AFMT_U8;	
static unsigned int sound_fragment = 8;
static unsigned int sound_fragment_size = 0;
static char * dsp_name = "/dev/dsp";

/*
 * Initialize /dev/dsp
 */
int sound_dsp_init(int mode) {
    printfv(3,"  setting %s...\n", dsp_name);

    /* Get sound-device */
    if ( (sound_handle = open(dsp_name, mode) ) < 0) {
	printfee("Can't open %s.", dsp_name);
	return 1;
    }

    /* set sound fragment size and number of fragments */
    sound_fragment = (log2(sound_bsize) - 1) | (sound_fragments << 16);
    if ( ioctl(sound_handle, SNDCTL_DSP_SETFRAGMENT, &sound_fragment) < 0 )
	printfee("ioctl: SNDCTL_DSP_SETFRAGMENT failed.");
    sound_fragments = sound_fragment >> 16;
    sound_fragment_size = 1 << (sound_fragment & 0x7fff);
    if( sound_fragment_size*2*sound_stereo < sound_bsize) {
	printfv(1, "  sound fragment size is not set big enough.\n");
    }

    /* set sound format */
    if ( ioctl(sound_handle, SNDCTL_DSP_SETFMT, &sound_fmt) < 0 )
	printfee("ioctl: SNDCTL_DSP_SETFMT failed");

    /* set stereo or mono */
    sound_stereo --;
    if ( ioctl(sound_handle, SNDCTL_DSP_STEREO, &sound_stereo) < 0 )
	printfee("ioctl: SNDCTL_DSP_STEREO failed");
    sound_stereo ++;
			
    /* set sample rate */
    if ( ioctl(sound_handle, SNDCTL_DSP_SPEED, &sound_sample_rate) < 0 )
	printfee("ioctl: SNDCTL_DSP_SPEED failed");

    /* set sample size (bits/sample/channel) */
    if ( ioctl(sound_handle, SNDCTL_DSP_SAMPLESIZE, &sound_bps) < 0)
	printfee("ioctl: SNDCTL_DSP_SAMPLESIZE failed");

    printfv(3,"    sound fragments    : %d\n", sound_fragments);
    printfv(3,"    sound fragment size: %d\n", sound_fragment_size);
    printfv(3,"    number of channels : %d\n", sound_stereo);
    printfv(3,"    sound buffer size  : %d\n", sound_bsize);
    printfv(3,"    sample rate        : %d\n", sound_sample_rate);
    printfv(3,"    sample size        : %d\n", sound_bps);
	
    return 0;
}


/*
 *  Update sample rate, mono/stero
 */
int sound_dsp_update() {
    alloc_sound();			/* might be freed */

    if( !sound_use[snddev_dsp_r] && !sound_use[snddev_dsp_w])
	return 0;
	
    if(sound_fork)
	sound_kill_process();

    ioctl(sound_handle, SNDCTL_DSP_RESET);
	
    sound_stereo --;			/* bring to 0, 1 for mono/stereo */
    if ( ioctl(sound_handle, SNDCTL_DSP_STEREO, &sound_stereo) < 0 )
	printfee("ioctl: SNDCTL_DSP_STEREO failed");
    sound_stereo ++;			/* bring back to nr. of channels */
		
    if ( ioctl(sound_handle, SNDCTL_DSP_SPEED, &sound_sample_rate) <0 )
	printfee("ioctl: SNDCTL_DSP_SPEED failed");


    if(sound_fork)
	sound_fork_process();

    return 0;
}


/*
 * Get sound from sound-card
 *
 * Jan Kujawa <kujawa@kallisti.montana.com> 
 *   helped a lot to do this better.
 */
int sound_dsp_read() {
    int i;
    audio_buf_info bi;
    int nr_read = sound_stereo * sound_bsize / sound_fragment_size;
    unsigned char sound_buffer[sound_fragment_size * 2 * sound_stereo];

    /* get number of available fragments */
    if( ioctl(sound_handle, SNDCTL_DSP_GETISPACE, &bi) < 0) {
	printfee("ioctl: SNDCTL_DSP_GETISPACE failed.");
    }

    /* read any extra sound data */
    for(i=0; i < bi.fragments-nr_read; i++ )
	read(sound_handle, sound_buffer, sound_fragment_size);

    /* read the sound data, that will be display */
    if( read(sound_handle, sound_buffer, sound_fragment_size * nr_read) < 0) {
	printfee("reading sound failed."); 
    }

    /* this should no longer be necessary */
    if(sound_sync)
    	ioctl(sound_handle, SNDCTL_DSP_RESET);   

    sound_convert(sound_buffer, sound_stereo, sound_bps);

    return 0;
}

int sound_dsp_write(unsigned char *data) {

    write(sound_handle, data, sound_stereo * sound_bsize * sound_bps);
    return 0;
}


int sound_dsp_exit() {
    close(sound_handle);
    return 0;
}



