#include "cthugha.h"
#include "sound.h"

#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

char * mixer_names[] = SOUND_DEVICE_NAMES;
int * mixer_volumes = NULL;
int mixer_nr_names = SOUND_MIXER_NRDEVICES;

static char * mixer_name = "/dev/mixer";

int mixer_init_volumes() {
    int i;

    if(mixer_volumes == NULL) {
	mixer_volumes = cth_memory(NULL, mixer_nr_names*sizeof(int), "");
	for(i=0; i < mixer_nr_names; i++)
	    mixer_volumes[i] = -1;
    }
    return 0;
}

int mixer_set_volume(char * name, int volume) {
    int i;

    mixer_init_volumes();

    for(i=0; i < mixer_nr_names; i++)
	if( strcasecmp(mixer_names[i], name) == 0) {
	    mixer_volumes[i] = volume;
	    i = mixer_nr_names + 1;
	}
    if(i == mixer_nr_names) {
	printfe("unknown mixer setting %s.\n", name);
	return 1;
    }
    if(strcasecmp(name, "cd") ==0)
	sound_use[snddev_cd] = 1;

    return 0;
}


/*
 * Initialize /dev/mixer
 */
int sound_set_mixer() {
    int mixer_des;
    int dev_mask = 0;
    int sound_mask = 0;
    int i;

    printfv(3,"  setting %s...\n", mixer_name);

    /* open mixer-file */	
    if ( (mixer_des = open(mixer_name, O_RDWR)) < 0) {
	printfee("Can not open %s.", mixer_name);
	return 1;
    }

    /* check, which features are actually available */
    if( ioctl(mixer_des, SOUND_MIXER_READ_DEVMASK, &dev_mask) < 0) {
	printfee("Can not get mixer device mask.");
    }
    for(i=0; i < mixer_nr_names; i++) {
	if( (dev_mask & 1) == 0) {
	    if(mixer_volumes[i] >= 0) 
		printfv(1, "    unsupported mixer setting %s.\n", 
			mixer_names[i]);
	    mixer_names[i] = NULL;
	    mixer_volumes[i] = -1;
	}
	dev_mask >>= 1;
    }

    /* create sound mask */
    for(i=0; i < mixer_nr_names; i++) {
	if( mixer_volumes[i] > 0) {
	    sound_mask |= (1 << i);
	} else if( mixer_volumes[i] == 0) {
	    printfv(3, "    disabling %s as input.\n", mixer_names[i]);
	}
    }
    
    if( sound_mask) {
	/* Select input sources */
	if(ioctl(mixer_des,MIXER_WRITE(SOUND_MIXER_RECSRC),&sound_mask) < 0) {
	    printfee("Can not set recording source");
	}

	/* Select input-volumes */
	for(i=0; i < mixer_nr_names; i++) {
	    if(mixer_volumes[i] >= 0) {
		ioctl(mixer_des, MIXER_WRITE(i), &mixer_volumes[i]);
		printfv(3,"    using %s as input (volume: %d).\n", 
			mixer_names[i],  mixer_volumes[i] & 255);
	    }
	}
    }

    /* close mixer-file */
    close( mixer_des);	
	
    return 0;
}

