/*
    CTHUGHA-L								sound.h
	
    Funktions that deal with sound.
	
*/

#ifndef __SOUND_H__
#define __SOUND_H__

#include "cth_buffer.h"
#include "feature.h"

/* 
 *  General stuff
 */
int init_sound();
int exit_sound();
int alloc_sound();
int free_sound();

typedef char char2[2];
extern char2 * sound_data;			/* Stereo-Data */
extern int sound_bsize;
extern int sound_fork;				/* read in an extra process */

extern int sine[320];

int sound_convert(unsigned char * data, int stereo, int bps);

/* 
 *  get-sound
 */
int get_sound();
extern int sound_minnoise;			/* quiet is below this */
extern int sound_quiet_change;			/* change after quiet-pause */
extern int sound_wait_quiet;			/* max. quiet interval */
extern int sound_peaklevel;			/* above that is loud */
extern int sound_wait;				/* time till change */
extern int sound_wait_beat;			/* peaks till change */
extern int sound_wait_random;			/* extra random time */
extern int sound_wait_min;			/* min time between change */
void sound_random();
extern int sound_sync;				/* sync after reading sound */
int sound_read();
int sound_fork_process();
int sound_kill_process();

/* 
 *  sound-processing
 */
int massage_audio();				/* audio-processing 1 */
int change_massage_style(int to);
extern int sound_massage_style;			/* massage style */
extern int massage_first;			/* massage to start with */

extern int sound_flashlight;
extern int sound_use_flashlight;
int flashlight();
int change_flashlight(int to);

int FFT();					/* audio-processing 2 */
int change_FFT(int to);
extern int sound_FFT;				/* use FFT or don't */
extern int sound_use_fft;			/* allow fft-usage */

/* 
 *  sound-display (wave-functions)
 */
extern feature waves;				/* possible wave-functions */
extern void (* display_wave)(void);		/* active wave-function */
int change_display_wave(int to);		/* select new wave-fkt */
int select_wave();
extern char wave_first[256];			/* Start with this wave */

int init_tables();
typedef unsigned char pal_table[256];		/* Table for display_wave */
#define MAX_TABLES	255			/* Max. nr. of tables */
extern pal_table tables[MAX_TABLES];		/* Palette-Tables */
extern int nr_tables;				/* number of tables */
extern int active_table;			/* active table */
extern int table_first;			/* table to start with */
int change_table(int to);


int init_sound_process();
int sound_noise_check();


/*
 * things for the mixer
 */
extern char * mixer_names[];
extern int * mixer_volumes;
extern int mixer_nr_names;
int mixer_set_volume(char * name, int volume);

int mixer_init_volumes();
int sound_set_mixer();

/*
 * functions and variables for reading from the different sound devices
 */
enum sound_dev_t {
    snddev_dsp_r = 0,
    snddev_dsp_w,
    snddev_mixer,
    snddev_cd,
    snddev_debug,
    snddev_sock,
    snddev_play,
    snddev_max
};
extern int sound_use[snddev_max];
int sound_use_clear();


/* read sound from soundcard (DSP) */
extern int sound_stereo;			/* number of channels */
extern int sound_bps;
extern int sound_sample_rate;		
extern int sound_fragments;

int sound_dsp_init(int mode);
int sound_dsp_read(void);
int sound_dsp_write(unsigned char * buffer);
int sound_dsp_update(void);
int sound_dsp_exit(void);


/* generate random sound */
int sound_random_read(void);


/* reading via network */
int sound_net_init(void);
int net_request(int req);
int sound_net_read(void);
int sound_net_exit(void);	
extern char sound_hostname[256];


/* read sound from a file and play */
extern char sound_file_name[512];
int sound_play_init(void);
int sound_play_read(void);
int sound_play_exit(void);


#endif



