#include "cthugha.h"
#include "sound.h"
#include "interface.h"
#include "options.h"
#include "display.h"

#include <math.h>

int sound_timer = 0;				/* used for automatic change */
int sound_minnoise = 30;			/* quiet is below this */
int sound_peaklevel = 240;			/* above that is loud */
int sound_wait_quiet = 500;			/* max. quiet interval (5 sec) 
						   then text is displayed */
int sound_quiet_change = 150;			/* change after quiet-pause 
						   1.5 seconds */

int sound_wait_beat = 16;			/* peaks till change */

int sound_wait_min = 500;			/* min time between change 
						   (5 sec) */
int sound_wait_random = 1000;			/* extra random wait-time 
						   (10 sec) */
int sound_wait = 1000;				/* time till change */

int sound_FFT = 0;				/* use FFT or don't */
int sound_use_fft = 1;				/* allow fft-usage */

int sound_massage_style = 1;			/* massage style */
int massage_first = CHANGE_RANDOM;		/* massage to start with */

pal_table tables[MAX_TABLES];			/* Palette-usage-tables */
int nr_tables = 0;
int active_table = 0;
int table_first = CHANGE_RANDOM;		/* table to start with */


int sine[320];					/* sine in 1/320 */
int Bsine[MAX_BUFF_WIDTH];			/* sine in 1/BUFF_WIDTH */
__complex__ float wp[MAX_BUFF_WIDTH];

/*
 * local variables
 */
static int sound_beats = 0;			/* loudness-count */
static int sound_quiet_since = 0;		/* used to determine if
						   quiet msg. should be disp.*/

int init_sound_FFT();


int init_sound_process() {

    init_sound_FFT();
    init_tables();
	
    if ( sound_wait_random <= 0)
	sound_wait_random = 1;

    /* set initial wait-time till change */
    sound_wait = sound_wait_min + rand() % sound_wait_random;

    return 0;
}
/*
 * Initalization for FFT 
 * the sine-table is also used by some other functions
 */
int init_sound_FFT() {
    int i; 
 
    for(i=0; i<320; i++) 
	sine[i]=(int)(128*sin((float)i*0.03927)); 
	
    for(i=0; i<BUFF_WIDTH; i++)
    	Bsine[i] = (int)(128*sin((float)i/((float)BUFF_WIDTH)*4.0*3.141592654));

    for(i=0; i < sound_bsize; i++) {
	__real__ wp[i] = cos(2.0*M_PI/(double)sound_bsize*(double)i);
	__imag__ wp[i] = sin(2.0*M_PI/(double)sound_bsize*(double)i);
    }
	
 
    return 0;
}


/*
 * Calculate the noisyness. And from that the time since when the 
 * system is silent and if it is time to change the display 
 * returns: 
 *   -1 silentce-message,
 *    1  it's time to change
 *    0 otherwise
 */

int sound_noise_check() {
    int x;
    char * sdata;
    int noisy;				/* there was some sound */
    int peaknoise;			/* it was loud */
    int high_sample;
    int low_sample;
    long now;


    /* Calculate weather we have noise or a peaklevel */
    sdata = (char *)sound_data;
    low_sample = high_sample = *sdata;
    noisy = 0, peaknoise = 0;
    for( x = sound_bsize * 2; x != 0; x--) {
	if ( *sdata > high_sample)
	    high_sample = *sdata;
	if ( *sdata < low_sample)
	    low_sample = *sdata;
	if ( (high_sample-low_sample) > sound_peaklevel) {
	    peaknoise = 1;	
	    noisy = 1;
	    break;
	} else if ( (high_sample-low_sample) > sound_minnoise) 
	    noisy = 1;
	sdata ++;
    }

    now = gettime();
	
    /* Check for interrupted silence */
    if ( sound_quiet_change)
	if ( noisy && ( (now - sound_quiet_since) > sound_quiet_change) ) {
	    sound_quiet_since = now;		/* from now on no not quiet */
	    return 1;				/* ret: interrupted silence */
	}
	
    /* Check for long quietness */
    if ( !noisy ) {				
	if( (now - sound_quiet_since) > sound_wait_quiet) {
	    sound_quiet_since = now;
	    return -1;				/* ret: long quiet */
	}
    } else
	sound_quiet_since = now;		/* not quiet now */
		
    /* Check for peaksound */
    if ( sound_wait_beat)
	if ( peaknoise ) {
	    if ( sound_beats ++ > sound_wait_beat) {
		sound_beats = 0;
		return 1;
	    }
	} 

    /* nothing special happend */
    if( (now - sound_timer) > sound_wait) {	/* same for long enough */
	sound_timer = now;
	sound_wait = sound_wait_min + rand() % max(1,sound_wait_random);
	return 1;
    }
    return 0;
}

/* Macros for compatibility with DOS-version */
#define stereo sound_data
#define massageStyle sound_massage_style

int massage_audio() {
    int temp,x; 
    int temp2; 
 
    switch (massageStyle) { 
    case 0: 
    default: 
	break; 
    case 1: 
	temp=stereo[0][1]; 
	temp2=stereo[0][0]; 
	for (x=1; x < sound_bsize; x++) { 
	    if ((stereo[x][1]-temp)>10) { 
		stereo[x][1]=temp+10; 
	    } else if ((stereo[x][1]-temp)<-10) { 
		stereo[x][1]=temp-10; 
	    } 
	    if ((stereo[x][0]-temp2)>10) { 
		stereo[x][0]=temp2+10; 
	    } else if ((stereo[x][0]-temp2)<-10) { 
		stereo[x][0]=temp2-10; 
	    } 
	    temp=stereo[x][1]; 
	    temp2=stereo[x][0]; 
	} 
	break; 
    case 2: 
	temp=stereo[0][1]; 
	temp2=stereo[0][0]; 
	for (x=1; x < sound_bsize; x++) { 
	    if ((stereo[x][1]-temp)>3) { 
		stereo[x][1]=temp+3; 
	    } else if ((stereo[x][1]-temp)<-3) { 
		stereo[x][1]=temp-3; 
	    } 
	    if ((stereo[x][0]-temp2)>3) { 
		stereo[x][0]=temp2+3; 
	    } else if ((stereo[x][0]-temp2)<-3) { 
		stereo[x][0]=temp2-3; 
	    } 
	    temp2=stereo[x][0]; 
	    temp=stereo[x][1]; 
	} 
	break; 
    case 3:			/* low pass filter */
	temp=stereo[0][1];
	temp2=stereo[0][0];
	for (x=1; x < sound_bsize; x++) {
	    stereo[x][1] = stereo[x-1][1] + (stereo[x][1] - temp) / 16;
	    stereo[x][0] = stereo[x-1][0] + (stereo[x][0] - temp2) / 16;
	    temp2=stereo[x][0];
	    temp=stereo[x][1];
	}
	break;
    } 
    return 0;
}

int change_massage_style(int to) {
    sound_massage_style = val_change(to, 0, 3, sound_massage_style);
    return 0;
}

/*****************************************************************************/

int sound_flashlight = 0;
int sound_use_flashlight = 1;

int flashlight() {
    int channel;
    unsigned char *p, *q;
    static palette Pal;
    static unsigned int slab[256];
    int dir,currentd,lastgot,i,lens,got, lastd=1,temp;
    int lastcapc,nextonec,amt;
    
    if( (sound_flashlight == 0) || (sound_use_flashlight == 0) )
	return 0;

    lens=0;
    for (i=0; i < 256; i++) 
	slab[i] = 0;

    lastgot=128;
    lens=0;
    lastd=1;
    lastcapc = 128;
    nextonec = 1;

    
    for (channel=0; channel<2; channel++)
	for (i=0; i < sound_bsize; i++) {
	    got=stereo[i][channel];
	    dir=got-lastgot;
	    
	    if (dir>1)
		currentd=1;
	    else if (dir<-1)
		currentd=-1;
	    else
		currentd=lastd;
	    
	    if (currentd!=lastd) {
		if (lens>255)
		    lens=255;
		lens=lens>>3;
		
		slab[lens]+=(lens>>1);
		
		lens=0;
		
		if (nextonec == 1) {
		    nextonec=0;
		    lastcapc=got;
		} else {
		    nextonec = 1;
		    amt=abs(lastcapc-got);
		    if (amt>127)
			amt=127;
		    slab[amt>>2]++;
		}
	    } else
		lens++;
	    
	    
	    lastd=currentd;
	    lastgot=got;
	}

    p = (unsigned char *)Pal;
    q = (unsigned char *)(palettes.entries[palettes.current].data);
    
    for (i=0; i < 256; i++) {
	temp=slab[(255-i)>>3];
	
	if (temp>6)
	    temp=6;
	
	q = (unsigned char *)(palettes.entries[
	    (palettes.current+temp)%palettes.nr_entries].data)+i*3;
	
	*p++ = *q++;
	*p++ = *q++;
	*p++ = *q++;
	
    }

    cth_setpalette(Pal, 1);

    return 0;
}

int change_flashlight(int to) {
    sound_flashlight = val_change(to, 0, sound_use_flashlight, sound_flashlight);
    return 0;
}
    


/*
 * compute FFT 
 */
static int r(int k, int n) {
    int r = 0;
    for(; n > 1; n >>= 1) {
        r = (r<<1) | (k&1);
        k >>= 1;
    }
    return r;
}

int FFT() {
    int channel;
    int h, k;
    __complex__ float c[sound_bsize];

    if( !sound_FFT || !sound_use_fft )
	return 0;

    for(channel=0; channel < 2; channel++) {
	int p;
	int z, zp;

	for(k=0; k < sound_bsize; k++) {
	    c[k] = (float)sound_data[k][channel];
	}

	p = sound_bsize / 2;
	z = 1;
	for(h = sound_bsize; h != 0; h >>= 1) {

	    zp = 0;
	    for(k=0; k < sound_bsize; k++) {
		if( (k & p) == 0) {
		    __complex__ float t = c[k] + c[k+p];
		    c[k+p] = (c[k] - c[k+p])*wp[zp];
		    c[k] = t;
		}
		zp = (zp + z) % sound_bsize;
	    }
	    p >>= 1;
	    z *= 2;
	}
	
	for(k=0; k < sound_bsize; k++) {
	    float l = __real__ c[k] * __real__ c[k] +
		__imag__ c[k] * __imag__ c[k];
	    int a = (int)sqrt( l / (float)sound_bsize); 
	    sound_data[r(k,sound_bsize)][channel] = (a > 255) ? 127 : a - 128;
	}
    }
	
    return 0;
}
 	

int change_FFT(int to) {
    sound_FFT = val_change(to, 0, sound_use_fft, sound_FFT);
    return 0;
}







