#include "../cthugha.h"
#include "../display.h"
#include "../disp-sys.h"

#include <vga.h>
#include <vgagl.h>


int screen_modes[] = {			/* allowed modes */
    G320x200x256,		/* 0 */
    G320x240x256,		/* 1 */
    G320x400x256,		/* 2 */
    G360x480x256,		/* 3 */
    G640x480x256,		/* 4 */
    G800x600x256,		/* 5  here it starts getting tiny */
    G1024x768x256,		/* 6 */
    G1280x1024x256,		/* 7 */
    G1600x1200x256,		/* 8  only for completeness */
};
int nr_screen_modes = sizeof(screen_modes) / sizeof(int);

GraphicsContext display_screen_context;
GraphicsContext display_virtual_context;

int display_copybox = 0;		/* use copybox. Not available
					   at some screen-modes */
int display_direct = 1;			/* draw directly to screen */

int display_syncwait = 0;

char display_font[ 256 * 8 * 8 ];

int init_display_sys() {
    vga_init(); 				/* initialize svgalib */
    return 0;
}


int init_graph_mode_sys() {
    vga_modeinfo * mi;

    if ( (display_mode >= nr_screen_modes) || (display_mode < 0))
	display_mode = 0;

    /* create screen-context */
    vga_setmode(screen_modes[ display_mode ]);		
    gl_setcontextvga(screen_modes[ display_mode ]);

    gl_getcontext(&display_screen_context);
/*    gl_enablepageflipping( &display_screen_context);  */
    /* direct display only in 320x200 */
    if ( (display_mode != 0) || (bpp != 8))
	display_direct = 0;

    mi = vga_getmodeinfo(screen_modes[ display_mode ]);

    /* can't use copybox if screen-mode is planar 256 colors */
    display_copybox = (mi->flags & 4) ? 0 : 1;
     
    /* prepare font */
    gl_expandfont(8,8,255, gl_font8x8, display_font);
    gl_setfont( 8,8, display_font);
    gl_setwritemode( WRITEMODE_MASKED);
    gl_colorfont(8,8, 255, display_font);

    /* create context for drawing in memory */
    gl_setcontextvgavirtual(screen_modes[ display_mode ]);
    gl_getcontext(&display_virtual_context);
/*    gl_enablepageflipping( &display_virtual_context);  */

    disp_size.x = WIDTH/BYTESPERPIXEL;
    disp_size.y = HEIGHT;
    bytes_per_line = WIDTH;
    
    if ( (disp_size.x < BUFF_WIDTH) || (disp_size.y < BUFF_HEIGHT) ) {
	exit_graph_mode();
	printfe("Screen must not be smaller than buffer.");
	return 1;
    }

    return 0;
}

int exit_graph_mode_sys() {
    vga_setmode(TEXT);
    return 0;
}


int disp_clear_box_sys(int x, int y, int width, int height) {
    gl_fillbox(x, y, width, height, 0);
    return 0;
}

int disp_copy_box_sys(int x, int y, int width, int height, 
		      int destx, int desty) {

    gl_copybox(x, y, width, height,  destx, desty);

    return 0;
}
char * pre_draw_sys() {
    if( display_direct )
	gl_setcontext( &display_screen_context);
    else
	gl_setcontext( &display_virtual_context);

    return VBUF;
}


int disp_copy_sys(int clear) {
    if( display_syncwait )
	vga_waitretrace();

    /* copy virtual screen to physical screen */
    if( !display_direct) {
	if( clear || !display_copybox)		/* draw full screen */
	    gl_copyscreen( &display_screen_context);
	else {					/* or only a part */
	    gl_setcontext( &display_screen_context);
	    gl_copyboxfromcontext( &display_virtual_context,
				  SCREEN_OFFSET_X, SCREEN_OFFSET_Y,
				  draw_size.x, draw_size.y,
				  SCREEN_OFFSET_X, SCREEN_OFFSET_Y);
	}
    }
    return 0;
}





/*
 * display text
 */
int display_print_string_sys(int x,int y,char* text) {
    gl_writen(x,y, min(strlen(text), WIDTH/8), text);
    return 0;
}
